"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const retrieve_validation_1 = __importDefault(require("./retrieve-validation"));
function validateText(textToBeValidated, options) {
    var _a, _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        if (!textToBeValidated) {
            throw new Error('You must pass in text to be validated');
        }
        if (typeof textToBeValidated !== 'string') {
            throw new Error('The text to be validated must be a string');
        }
        if (options) {
            const allowedMediums = [
                'all',
                'braille',
                'embossed',
                'handheld',
                'print',
                'projection',
                'screen',
                'speech',
                'tty',
                'tv',
            ];
            if (options.medium && !allowedMediums.includes(options.medium)) {
                throw new Error(`The medium must be one of the following: ${allowedMediums.join(', ')}`);
            }
            const allowedWarningLevels = [0, 1, 2, 3];
            if (options.warningLevel && !allowedWarningLevels.includes(options.warningLevel)) {
                throw new Error(`The warning level must be one of the following: ${allowedWarningLevels.join(', ')}`);
            }
        }
        const params = {
            text: encodeURIComponent(textToBeValidated),
            usermedium: (_a = options === null || options === void 0 ? void 0 : options.medium) !== null && _a !== void 0 ? _a : 'all',
            warning: (options === null || options === void 0 ? void 0 : options.warningLevel) ? options.warningLevel - 1 : 'no',
            output: 'application/json',
            profile: 'css3',
        };
        const url = `https://jigsaw.w3.org/css-validator/validator?${Object.entries(params)
            .map(([key, val]) => `${key}=${val}`)
            .join('&')}`;
        const cssValidationResponse = yield retrieve_validation_1.default(url);
        const base = {
            valid: false,
            errors: [],
        };
        const result = (options === null || options === void 0 ? void 0 : options.warningLevel)
            ? Object.assign(Object.assign({}, base), { warnings: [] }) : base;
        result.valid = cssValidationResponse.validity;
        (_b = cssValidationResponse.errors) === null || _b === void 0 ? void 0 : _b.forEach((error) => {
            result.errors.push({
                line: error.line,
                message: error.message,
            });
        });
        if ('warnings' in result) {
            (_c = cssValidationResponse.warnings) === null || _c === void 0 ? void 0 : _c.forEach((warning) => {
                result.warnings.push({
                    line: warning.line,
                    message: warning.message,
                    level: (warning.level + 1),
                });
            });
        }
        return result;
    });
}
exports.default = validateText;
//# sourceMappingURL=validate-text.js.map